from typing import Tuple
from uuid import UUID, uuid4

from django.conf import settings
from django.db import models
from django.utils.translation import gettext as _
from sinp_nomenclatures.models import Nomenclature

from .managers import OrganismManager, OrganismMemberManager
from .validators import phone_regex


class BaseModel(models.Model):
    """Generic base model with standard metadatas"""

    timestamp_create = models.DateTimeField(auto_now_add=True, editable=False)
    timestamp_update = models.DateTimeField(auto_now=True, editable=False)

    class Meta:
        abstract = True


# Create your models here.
class Organism(BaseModel):
    """Organism model"""

    uuid = models.UUIDField(
        default=uuid4,
        unique=True,
        editable=False,
        verbose_name=_("Identifiant unique"),
    )
    administrative_reference = models.CharField(
        max_length=100, blank=True, verbose_name=_("SIRET")
    )
    parent = models.ForeignKey(
        "Organism",
        verbose_name=_("Organisme parent"),
        on_delete=models.SET_NULL,
        null=True,
        blank=True,
    )
    label = models.CharField(
        max_length=500, unique=True, verbose_name=_("Nom")
    )
    short_label = models.CharField(
        max_length=50, unique=True, verbose_name=_("Nom court")
    )
    action_scope = models.ForeignKey(
        Nomenclature,
        on_delete=models.DO_NOTHING,
        limit_choices_to={"type__mnemonic": "action_scope"},
        related_name="organism_action_scope",
        verbose_name=_("Périmètre d'action"),
        help_text=_("Périmètre d'action de l'organisme"),
    )
    geographic_area = models.ManyToManyField(
        Nomenclature,
        blank=True,
        limit_choices_to={"type__mnemonic": "geographic_area"},
        related_name="organism_geographic_area",
        verbose_name=_("Zone géographique"),
        help_text=_("Zone d'action géographique de l'organisme"),
    )
    geographic_area_details = models.CharField(
        max_length=500,
        blank=True,
        verbose_name=_("Détails sur la zone géographique"),
        help_text=_("Information précisant la zone géographique d'action."),
    )
    status = models.ForeignKey(
        Nomenclature,
        on_delete=models.DO_NOTHING,
        limit_choices_to={"type__mnemonic": "organism_status"},
        related_name="organism_status",
        verbose_name=_("Statut"),
        help_text=_("Permet d'indiquer si l'organisme est public ou privé"),
    )
    type = models.ForeignKey(
        Nomenclature,
        on_delete=models.DO_NOTHING,
        limit_choices_to={"type__mnemonic": "organism_type"},
        related_name="organism_type",
        verbose_name=_("Type d'organisme"),
    )
    address = models.CharField(
        max_length=500, blank=True, verbose_name=_("Adresse")
    )
    postal_code = models.CharField(
        max_length=20, blank=True, verbose_name=_("Code postal")
    )
    municipality = models.CharField(
        max_length=250, blank=True, verbose_name=_("Commune")
    )
    email = models.EmailField(blank=True, verbose_name=_("Adresse mail"))
    phone_number = models.CharField(
        validators=[phone_regex],
        max_length=17,
        blank=True,
        verbose_name=_("Numéro de téléphone"),
    )
    url = models.URLField(max_length=200, blank=True, verbose_name=_("URL"))
    members = models.ManyToManyField(
        settings.AUTH_USER_MODEL,
        through="OrganismMember",
        through_fields=("organism", "member"),
        blank=True,
        related_name="organism_member",
        verbose_name=_("Membres"),
    )
    enabled = models.BooleanField(default=True, verbose_name=_("Actif"))
    extra_data = models.JSONField(
        blank=True, null=True, verbose_name=_("Additional datas")
    )
    objects = OrganismManager()

    class Meta:
        verbose_name_plural = _("organismes")

        constraints = [
            models.UniqueConstraint(
                fields=["uuid"],
                name="unique_uuid",
            ),
        ]

    def natural_key(self):
        return (self.uuid,)

    def __str__(self):
        return str(self.short_label)


class OrganismMember(BaseModel):
    """Organism members model"""

    member = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        on_delete=models.CASCADE,
        verbose_name=_("Utilisateur"),
        # related_name="organisms",
    )
    organism = models.ForeignKey(
        "Organism",
        on_delete=models.CASCADE,
        verbose_name=_("Organisme"),
        # related_name="members_set",
    )
    member_level = models.ForeignKey(
        Nomenclature,
        on_delete=models.CASCADE,
        limit_choices_to={"type__mnemonic": "member_level"},
        related_name="member_level",
        verbose_name=_("Niveau du membre"),
    )
    objects = OrganismMemberManager()

    class Meta:
        verbose_name_plural = _("Membre des organismes")
        unique_together = ("member", "organism", "member_level")

    def __str__(self):
        return f"{self.member} [{self.member_level}]"

    def natural_key(self) -> Tuple[UUID, UUID, str, str]:
        """_summary_

        Returns:
            Tuple[UUID,UUID, UUID,str,str]: A tuple with respectively
            member UUID, organism UUID, member_level code
        """
        return (
            self.member.username,
            self.organism.uuid,
            self.member_level.code,
            self.member_level.type.mnemonic,
        )



from django.urls import include, path
from rest_framework.routers import SimpleRouter

from .views import OrganismMemberViewset, OrganismViewset

app_name = "sinp_organisms"

router = SimpleRouter(trailing_slash=False)

router.register(r"organisms", OrganismViewset)
router.register(r"members", OrganismMemberViewset)

urlpatterns = [path("organisms/", include(router.urls))]



import logging

# from django.contrib.auth.mixins import LoginRequiredMixin
from rest_framework.permissions import DjangoModelPermissions
from rest_framework.viewsets import ModelViewSet

from .mixins import OrganismsFilteringMixin
from .models import Organism, OrganismMember
from .serializers import (
    OrganismDetailledSerializer,
    OrganismMemberSerializer,
    OrganismSerializer,
)

logger = logging.getLogger(__name__)


class OrganismViewset(OrganismsFilteringMixin, ModelViewSet):
    # serializer_class = OrganismDetailledSerializer
    permission_classes = [DjangoModelPermissions]
    queryset = (
        Organism.objects.select_related("action_scope")
        .select_related("parent")
        .select_related("status")
        .select_related("type")
        .select_related("parent")
        .prefetch_related("geographic_area")
        .prefetch_related("members")
        .all()
    )

    def get_serializer_class(self):
        depth = self.request.query_params.get("depth", 0)
        if depth == "1":
            return OrganismDetailledSerializer
        return OrganismSerializer


class OrganismMemberViewset(ModelViewSet):
    serializer_class = OrganismMemberSerializer
    permission_classes = [DjangoModelPermissions]
    queryset = (
        OrganismMember.objects.select_related("member")
        .select_related("member_level")
        .all()
    )



from django.contrib import admin
from django.utils.translation import gettext as _

# Register your models here.
from .models import Organism, OrganismMember


@admin.action(description=_("Mark selected items as active"))
def activate(_modeladmin, _request, queryset):
    """Set item active"""
    queryset.update(enabled=True)


@admin.action(description=_("Mark selected items as inactive"))
def inactivate(_modeladmin, _request, queryset):
    """Set item inactive"""
    queryset.update(enabled=False)


class OrganismAdmin(admin.ModelAdmin):
    """Organism admin viewset"""

    list_display = (
        "id",
        "short_label",
        "status",
        "type",
        "action_scope",
        "enabled",
        "parent",
    )
    list_filter = ("type", "enabled", "geographic_area")
    actions = [activate, inactivate]


class OrganismMemberAdmin(admin.ModelAdmin):
    """Organism members admin viewset"""

    list_display = (
        "member",
        "organism",
        "member_level",
    )
    list_filter = ("organism", "member_level")


admin.site.register(Organism, OrganismAdmin)
admin.site.register(OrganismMember, OrganismMemberAdmin)



# Generated by Django 4.2.11 on 2024-04-22 13:19

import uuid

import django.core.validators
import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("sinp_nomenclatures", "0001_initial"),
    ]

    operations = [
        migrations.CreateModel(
            name="Organism",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("timestamp_create", models.DateTimeField(auto_now_add=True)),
                ("timestamp_update", models.DateTimeField(auto_now=True)),
                (
                    "uuid",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        unique=True,
                        verbose_name="Identifiant unique",
                    ),
                ),
                (
                    "administrative_reference",
                    models.CharField(
                        blank=True, max_length=100, verbose_name="SIRET"
                    ),
                ),
                (
                    "label",
                    models.CharField(
                        max_length=500, unique=True, verbose_name="Nom"
                    ),
                ),
                (
                    "short_label",
                    models.CharField(
                        max_length=50, unique=True, verbose_name="Nom court"
                    ),
                ),
                (
                    "geographic_area_details",
                    models.CharField(
                        blank=True,
                        help_text=(
                            "Information précisant "
                            "la zone géographique d'action."
                        ),
                        max_length=500,
                        verbose_name="Détails sur la zone géographique",
                    ),
                ),
                (
                    "address",
                    models.CharField(
                        blank=True, max_length=500, verbose_name="Adresse"
                    ),
                ),
                (
                    "postal_code",
                    models.CharField(
                        blank=True, max_length=20, verbose_name="Code postal"
                    ),
                ),
                (
                    "municipality",
                    models.CharField(
                        blank=True, max_length=250, verbose_name="Commune"
                    ),
                ),
                (
                    "email",
                    models.EmailField(
                        blank=True, max_length=254, verbose_name="Adresse mail"
                    ),
                ),
                (
                    "phone_number",
                    models.CharField(
                        blank=True,
                        max_length=17,
                        validators=[
                            django.core.validators.RegexValidator(
                                message=(
                                    "Les numéros de téléphones doivent "
                                    "être renseignés avec le format :"
                                    "'+999999999'. jusqu'à 15 chiffres "
                                    "sont autorisés"
                                ),
                                regex="^\\+?1?\\d{9,15}$",
                            )
                        ],
                        verbose_name="Numéro de téléphone",
                    ),
                ),
                ("url", models.URLField(blank=True, verbose_name="URL")),
                (
                    "enabled",
                    models.BooleanField(default=True, verbose_name="Actif"),
                ),
                (
                    "extra_data",
                    models.JSONField(
                        blank=True, null=True, verbose_name="Additional datas"
                    ),
                ),
                (
                    "action_scope",
                    models.ForeignKey(
                        help_text="Périmètre d'action de l'organisme",
                        limit_choices_to={"type__mnemonic": "action_scope"},
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="organism_action_scope",
                        to="sinp_nomenclatures.nomenclature",
                        verbose_name="Périmètre d'action",
                    ),
                ),
                (
                    "geographic_area",
                    models.ManyToManyField(
                        blank=True,
                        help_text="Zone d'action géographique de l'organisme",
                        limit_choices_to={"type__mnemonic": "geographic_area"},
                        related_name="organism_geographic_area",
                        to="sinp_nomenclatures.nomenclature",
                        verbose_name="Zone géographique",
                    ),
                ),
            ],
            options={
                "verbose_name_plural": "organismes",
            },
        ),
        migrations.CreateModel(
            name="OrganismMember",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("timestamp_create", models.DateTimeField(auto_now_add=True)),
                ("timestamp_update", models.DateTimeField(auto_now=True)),
                (
                    "member",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to=settings.AUTH_USER_MODEL,
                        verbose_name="Utilisateur",
                    ),
                ),
                (
                    "member_level",
                    models.ForeignKey(
                        limit_choices_to={"type__mnemonic": "member_level"},
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="member_level",
                        to="sinp_nomenclatures.nomenclature",
                        verbose_name="Niveau du membre",
                    ),
                ),
                (
                    "organism",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="sinp_organisms.organism",
                        verbose_name="Organisme",
                    ),
                ),
            ],
            options={
                "verbose_name_plural": "Membre des organismes",
                "unique_together": {("member", "organism", "member_level")},
            },
        ),
        migrations.AddField(
            model_name="organism",
            name="members",
            field=models.ManyToManyField(
                blank=True,
                related_name="organism_member",
                through="sinp_organisms.OrganismMember",
                to=settings.AUTH_USER_MODEL,
                verbose_name="Membres",
            ),
        ),
        migrations.AddField(
            model_name="organism",
            name="parent",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="sinp_organisms.organism",
                verbose_name="Organisme parent",
            ),
        ),
        migrations.AddField(
            model_name="organism",
            name="status",
            field=models.ForeignKey(
                help_text=(
                    "Permet d'indiquer si l'organisme est public ou privé"
                ),
                limit_choices_to={"type__mnemonic": "organism_status"},
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="organism_status",
                to="sinp_nomenclatures.nomenclature",
                verbose_name="Statut",
            ),
        ),
        migrations.AddField(
            model_name="organism",
            name="type",
            field=models.ForeignKey(
                limit_choices_to={"type__mnemonic": "organism_type"},
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="organism_type",
                to="sinp_nomenclatures.nomenclature",
                verbose_name="Type d'organisme",
            ),
        ),
        migrations.AddConstraint(
            model_name="organism",
            constraint=models.UniqueConstraint(
                fields=("uuid",), name="unique_uuid"
            ),
        ),
    ]



import uuid

from django.contrib.auth import get_user_model
from django.test import TestCase
from sinp_nomenclatures.models import Nomenclature

from .models import Organism, OrganismMember

User = get_user_model()

org1_uuid = uuid.uuid4()


class OrganismTestCase(TestCase):
    fixtures = [
        "nomenclatures.json",
    ]

    def setUp(self):
        self.user1 = User.objects.create(
            username="user1", email="user1@test.com", password="user1Pwd!"
        )
        self.user2 = User.objects.create(
            username="user2", email="user2@test.com", password="user2Pwd!"
        )
        self.org1 = Organism.objects.create(
            label="Organism 1",
            short_label="ORG1",
            uuid=org1_uuid,
            action_scope=Nomenclature.objects.get(
                type__mnemonic="action_scope", code="reg"
            ),
            status=Nomenclature.objects.get(
                type__mnemonic="organism_status", code="pub"
            ),
            type=Nomenclature.objects.get(
                type__mnemonic="organism_type", code="pubestab"
            ),
        )
        self.org_member1 = OrganismMember.objects.create(
            member=self.user1,
            organism=self.org1,
            member_level=Nomenclature.objects.get(
                type__mnemonic="member_level", code="manager"
            ),
        )

    def test_organisms(self):
        """Animals that can speak are correctly identified"""
        org1_1 = Organism.objects.get(short_label="ORG1")
        org1_2 = Organism.objects.get_by_natural_key(*(org1_uuid,))
        user1 = User.objects.get(username="user1")
        member1 = OrganismMember.objects.get_by_natural_key(
            *(
                "user1",
                org1_uuid,
                "manager",
                "member_level",
            )
        )
        user2 = User.objects.get_by_natural_key(
            username="user2",
        )
        self.assertEqual(str(org1_1), "ORG1")
        self.assertEqual(org1_1.uuid, org1_uuid)
        self.assertEqual(org1_1.natural_key(), (org1_uuid,))
        self.assertIn(user1, org1_1.members.all())
        self.assertNotIn(user2, org1_1.members.all())
        self.assertEqual(org1_1, org1_2)
        self.assertEqual(user1, member1.member)
        self.assertIn(member1, org1_1.organismmember_set.all())
        self.assertEqual(str(member1), f"{user1} [Gestionnaire]")
        self.assertEqual(
            member1.natural_key(),
            (user1.username, org1_1.uuid, "manager", "member_level"),
        )



#!/usr/bin/python

"""SINP Organisms validators
"""

from django.core.validators import RegexValidator

phone_regex = RegexValidator(
    regex=r"^\+?1?\d{9,15}$",
    message=(
        "Les numéros de téléphones doivent être renseignés avec le format :"
        "'+999999999'. jusqu'à 15 chiffres sont autorisés"
    ),
)



from rest_framework.serializers import ModelSerializer
from sinp_nomenclatures.serializers import NomenclatureSerializer

from .models import Organism, OrganismMember


class OrganismMemberSerializer(ModelSerializer):
    member_level = NomenclatureSerializer(read_only=True)

    class Meta:
        model = OrganismMember
        fields = [
            "id",
            "member",
            "member_level",
        ]


class OrganismSerializer(ModelSerializer):
    class Meta:
        model = Organism
        fields = [
            "id",
            "uuid",
            "label",
            "short_label",
            "action_scope",
            "geographic_area",
            "geographic_area_details",
            "status",
            "type",
            "address",
            "postal_code",
            "municipality",
            "email",
            "phone_number",
            "url",
        ]
        read_only_fields = [
            "uuid",
        ]


class OrganismDetailledSerializer(OrganismSerializer):
    action_scope = NomenclatureSerializer(read_only=True)
    geographic_area = NomenclatureSerializer(read_only=True, many=True)
    status = NomenclatureSerializer(read_only=True)
    type = NomenclatureSerializer(read_only=True)
    # members = OrganismMemberSerializer(many=True)



from uuid import UUID

from django.db.models import Manager


class OrganismManager(Manager):
    """Organisms custom manager"""

    def get_by_natural_key(self, uuid: UUID):
        """Get by natural key function for organisms

        Args:
            uuid (_type_): _description_

        Returns:
            _type_: _description_
        """
        return self.get(uuid=uuid)


class OrganismMemberManager(Manager):

    def get_by_natural_key(
        self,
        member_username: str,
        organism_uuid: UUID,
        member_level_code: str,
        member_level_type_code: str,
    ):
        return self.get(
            member__username=member_username,
            organism__uuid=organism_uuid,
            member_level__code=member_level_code,
            member_level__type__mnemonic=member_level_type_code,
        )
